// This file is part of mzCalc.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzCalc. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#ifndef FILLBATTLE_H
#define FILLBATTLE_H

#include "verbosity.h"

#include <stdexcept>
#include <vector>

struct Battle;
struct Stats;

struct TokenizerException : public std::runtime_error {
    explicit TokenizerException(const std::string& what) : std::runtime_error(what) { }
    explicit TokenizerException(const char* what)        : std::runtime_error(what) { }
};

struct Tokenizer {
    explicit Tokenizer(const std::vector<std::string>& tokens)
        : tokens(tokens)
        , index(0)
    { }
    explicit Tokenizer(std::vector<std::string>&& tokens)
        : index(0)
    {
        std::swap(this->tokens, tokens);
    }
    Tokenizer& operator=(const Tokenizer& other) {
        tokens = other.tokens;
        index  = other.index;
        return *this;
    }
    Tokenizer& operator=(Tokenizer&& other) {
        std::swap(tokens, other.tokens);
        std::swap(index,  other.index);
        return *this;
    }

    bool hasNext() const;
    const std::string& next();

    std::vector<std::string> tokens;
    std::size_t index = 0;
};

bool fillBattle(Battle& battle,
                const Stats& stats,
                const std::vector<std::string> tokens,
                Verbosity verbosity = Verbosity::Warning);

#endif /* FILLBATTLE_H */
