// This file is part of mzCalc.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzCalc. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#ifndef STATS_H
#define STATS_H

#include "shiptype.h"

#include "maybe.h"

#include "defines.h"

#include <unordered_map>
#include <string>
#include <vector>

struct Stats {
    typedef std::pair<const std::string, ShipType> Pair;

    implicit Stats()
    { }

    inline const Maybe<const ShipType> operator[](const std::string& key) const {
        auto iter = ships.find(key);
        return Maybe<const ShipType>(&iter->second,
                                     iter != ships.end());
    }
    inline Maybe<ShipType> operator[](const std::string& key) {
        auto iter = ships.find(key);
        return Maybe<ShipType>(&iter->second,
                               iter != ships.end());
    }
    inline Maybe<ShipType> operator[](std::string&& key) {
        auto iter = ships.find(key);
        return Maybe<ShipType>(&iter->second,
                               iter != ships.end());
    }

    Maybe<ShipType> create(const std::string& name,
                           Class clazz,
                           const std::array<Class, MAX_TARGETS>& targets,
                           GunType type,
                           Init init,
                           Guns guns,
                           Armor armor,
                           ERes eres,
                           Cost cost,
                           Race race);

    inline size_t size() const {
        return ships.size();
    }

    inline std::unordered_map<std::string, ShipType>::const_iterator cbegin() const {
        return ships.cbegin();
    }
    inline std::unordered_map<std::string, ShipType>::iterator begin() {
        return ships.begin();
    }
    inline std::unordered_map<std::string, ShipType>::const_iterator cend() const {
        return ships.cend();
    }
    inline std::unordered_map<std::string, ShipType>::iterator end() {
        return ships.end();
    }

    std::vector<ShipType*> findCombatShips(const Race& race = Race::None, const Class& clazz = Class::None);

private:
    std::unordered_map<std::string, ShipType> ships;
};

#endif /* STATS_H */
