// This file is part of mzCalc.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License version 2, as published by the
// Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// mzCalc. If not, see <http://www.gnu.org/licenses/>.
//
// This work is Copyright (C) 2016 by Josse van der Plaat
//
// Individual portions may be copyright by individual contributors, and are
// included in this collective work with permission of the copyright owners.

#include "stats.h"

Maybe<ShipType> Stats::create(const std::string& name,
                              Class clazz,
                              const std::array<Class, MAX_TARGETS>& targets,
                              GunType type,
                              Init init,
                              Guns guns,
                              Armor armor,
                              ERes eres,
                              Cost cost,
                              Race race) {
    auto p = ships.emplace(std::piecewise_construct,
                           std::forward_as_tuple(name),
                           std::forward_as_tuple(name,
                                                 clazz,
                                                 targets,
                                                 type,
                                                 init,
                                                 guns,
                                                 armor,
                                                 eres,
                                                 cost,
                                                 race));
    return Maybe<ShipType>(&p.first->second, p.second);
}

std::vector<ShipType*> Stats::findCombatShips(const Race& race, const Class& clazz) {
    std::vector<ShipType*> combatShips;
    for (Stats::Pair& pair : ships) {
        ShipType& ship = pair.second;
        if ((ship.clazz == clazz || clazz == Class::None) &&
            (race == Race::None || ship.race == race) &&
            ship.type != GunType::Pod &&
            ship.type != GunType::StructureKill)
        {
            combatShips.push_back(&ship);
        }
    }
    return combatShips;
}
